###################################################
### chunk number 1: 
###################################################
library(Hmisc)
options(width=70,continue=" ")


###################################################
### chunk number 2: emdbook
###################################################
library(emdbook)


###################################################
### chunk number 3: bbsim
###################################################
set.seed(1001)
x1 = rbetabinom(n=1000,prob=0.1,size=50,theta=10)


###################################################
### chunk number 4: bbmle
###################################################
library(bbmle)


###################################################
### chunk number 5: likfun1
###################################################
mtmp <- function(prob,size,theta) {
  -sum(dbetabinom(x1,prob,size,theta,log=TRUE))
}


###################################################
### chunk number 6: fit1
###################################################
(m0 <- mle2(mtmp,start=list(prob=0.2,theta=9),data=list(size=50)))


###################################################
### chunk number 7: sum1
###################################################
summary(m0)


###################################################
### chunk number 8: prof1
###################################################
p0 <- profile(m0)


###################################################
### chunk number 9: confint1
###################################################
confint(p0)
confint(m0,method="quad")
confint(m0,method="uniroot")


###################################################
### chunk number 10: profplot1
###################################################
par(mfrow=c(1,2))
plot(p0,plot.confstr=TRUE)


###################################################
### chunk number 11: fit2
###################################################
m0f <- mle2(x1~dbetabinom(prob,size=50,theta),
            start=list(prob=0.2,theta=9),data=data.frame(x1))


###################################################
### chunk number 12: fit2f
###################################################
m0cf <- mle2(x1~dbetabinom(prob=plogis(lprob),size=50,theta=exp(ltheta)),
            start=list(lprob=0,ltheta=2),data=data.frame(x1))
confint(m0cf,method="uniroot")
confint(m0cf,method="spline")


###################################################
### chunk number 13: aod
###################################################
library(aod)
data(orob1)


###################################################
### chunk number 14: aodlikfun
###################################################
ML1 <- function(prob1,prob2,prob3,theta,x) {
  prob <- c(prob1,prob2,prob3)[as.numeric(x$dilution)]
  size <- x$n
  -sum(dbetabinom(x$y,prob,size,theta,log=TRUE))
}


###################################################
### chunk number 15: crowdertab
###################################################
crowder.results <- matrix(c(0.132,0.871,0.839,78.424,0.027,0.028,0.032,-34.991,
                            rep(NA,7),-34.829,
                            rep(NA,7),-56.258),
                          dimnames=list(c("prop diffs","full model","homog model"),
                            c("prob1","prob2","prob3","theta","sd.prob1","sd.prob2","sd.prob3","NLL")),
                          byrow=TRUE,nrow=3)
latex(crowder.results,file="",table.env=FALSE,title="model")


###################################################
### chunk number 16: aodfit1
###################################################
(m1 <- mle2(ML1,start=list(prob1=0.5,prob2=0.5,prob3=0.5,theta=1),
    data=list(x=orob1)))


###################################################
### chunk number 17:  eval=FALSE
###################################################
## ## would prefer ~dilution-1, but problems with starting values ...
## (m1B <- mle2(y~dbetabinom(prob,size=n,theta),
##              param=list(prob~dilution),
##              start=list(prob=0.5,theta=1),
##     data=orob1))


###################################################
### chunk number 18: aodfit2
###################################################
(m2 <- mle2(ML1,start=as.list(coef(m1)),
          control=list(parscale=coef(m1)),
          data=list(x=orob1)))


###################################################
### chunk number 19: aodprof2
###################################################
p2 <- profile(m2,prof.upper=c(Inf,Inf,Inf,theta=2000))


###################################################
### chunk number 20: aodstderr
###################################################
round(sqrt(diag(vcov(m2))),3)


###################################################
### chunk number 21: aodvar
###################################################
sqrt(1/(1+coef(m2)["theta"]))


###################################################
### chunk number 22: 
###################################################
sqrt(deltavar(sqrt(1/(1+theta)),meanval=coef(m2)["theta"],
         vars="theta",Sigma=vcov(m2)[4,4]))


###################################################
### chunk number 23: 
###################################################
m2b <- mle2(y~dbetabinom(prob,size=n,theta=1/sigma^2-1),
            data=orob1,
            parameters=list(prob~dilution,sigma~1),
            start=list(prob=0.5,sigma=0.1))
round(sqrt(diag(vcov(m2b))),3)["sigma"]
p2b <- profile(m2b,prof.lower=c(-Inf,-Inf,-Inf,0))


###################################################
### chunk number 24: 
###################################################
r1 <- rbind(confint(p2)["theta",],
            confint(m2,method="quad")["theta",])
rownames(r1) <- c("spline","quad")
r1


###################################################
### chunk number 25: 
###################################################
plot(p2,which="theta",plot.confstr=TRUE)


###################################################
### chunk number 26: 
###################################################
plot(p2b,which="sigma",plot.confstr=TRUE,
     show.points=TRUE)


###################################################
### chunk number 27: 
###################################################
ml0 <- function(prob,theta,x) {
  size <- x$n
  -sum(dbetabinom(x$y,prob,size,theta,log=TRUE))
}
m0 <- mle2(ml0,start=list(prob=0.5,theta=100),
          data=list(x=orob1))


###################################################
### chunk number 28: 
###################################################
logLik(m0)


###################################################
### chunk number 29: 
###################################################
m0f <- mle2(y~dbetabinom(prob,size=n,theta),
            parameters=list(prob~1,theta~1),
            data=orob1,
            start=list(prob=0.5,theta=100))
m2f <- mle2(y~dbetabinom(prob,size=n,theta),
            parameters=list(prob~dilution,theta~1),
            data=orob1,
            start=list(prob=0.5,theta=78.424))
m3f <- mle2(y~dbetabinom(prob,size=n,theta),
            parameters=list(prob~dilution,theta~dilution),
            data=orob1,
            start=list(prob=0.5,theta=78.424))


###################################################
### chunk number 30: 
###################################################
anova(m0f,m2f,m3f)


###################################################
### chunk number 31: 
###################################################
AICtab(m0f,m2f,m3f,weights=TRUE,delta=TRUE,sort=TRUE)
BICtab(m0f,m2f,m3f,delta=TRUE,nobs=nrow(orob1),sort=TRUE,weights=TRUE)
AICctab(m0f,m2f,m3f,delta=TRUE,nobs=nrow(orob1),sort=TRUE,weights=TRUE)


###################################################
### chunk number 32:  eval=FALSE
###################################################
## function (x, levels, conf = c(99, 95, 90, 80, 50)/100, nseg = 50,
##           absVal = TRUE, ...) {}


###################################################
### chunk number 33:  eval=FALSE
###################################################
## function (x, levels, which=1:p, conf = c(99, 95, 90, 80, 50)/100, nseg = 50,
##           plot.confstr = FALSE, confstr = NULL, absVal = TRUE, add = FALSE,
##           col.minval="green", lty.minval=2,
##           col.conf="magenta", lty.conf=2,
##           col.prof="blue", lty.prof=1,
##           xlabs=nm, ylab="score",
##           show.points=FALSE,
##           main, xlim, ylim, ...) {}


###################################################
### chunk number 34: 
###################################################
frogdat <- data.frame(
  size=rep(c(9,12,21,25,37),each=3),
  killed=c(0,2,1,3,4,5,rep(0,4),1,rep(0,4)))
frogdat$initial <- rep(10,nrow(frogdat))


###################################################
### chunk number 35: 
###################################################
library(ggplot2)


###################################################
### chunk number 36: 
###################################################
gg1 <- ggplot(frogdat,aes(x=size,y=killed))+geom_point()+
      stat_sum(aes(size=factor(..n..)))+
      labs(size="#")+scale_x_continuous(limits=c(0,40))


###################################################
### chunk number 37: 
###################################################
m3 <- mle2(killed~dbinom(prob=c*(size/d)^g*exp(1-size/d),
  size=initial),data=frogdat,start=list(c=0.5,d=5,g=1))
pdat <- data.frame(size=1:40,initial=rep(10,40))
pdat1 <- data.frame(pdat,killed=predict(m3,newdata=pdat))


###################################################
### chunk number 38: 
###################################################
m4 <- mle2(killed~dbinom(prob=c*((size/d)*exp(1-size/d))^g,
  size=initial),data=frogdat,start=list(c=0.5,d=5,g=1))
pdat2 <- data.frame(pdat,killed=predict(m4,newdata=pdat))


###################################################
### chunk number 39: 
###################################################
print(gg1 + geom_line(data=pdat1,colour="red")+geom_line(data=pdat2,colour="blue"))


###################################################
### chunk number 40: 
###################################################
coef(m4)
prof4 <- profile(m4)


###################################################
### chunk number 41: 
###################################################
print(plot(prof4))


###################################################
### chunk number 42: latticeprof
###################################################
prof4_df <- as.data.frame(prof4)
library(lattice)
print(xyplot(abs(z)~focal|param,data=prof4_df,
       subset=abs(z)<3,
       type="b",
       xlab="",
       ylab=expression(paste(abs(z),
           " (square root of ",Delta," deviance)")),
       scale=list(x=list(relation="free")),
             layout=c(3,1)))


###################################################
### chunk number 43: ggplotprof
###################################################
print(ggplot(subset(prof4_df,abs(z)<3),
       aes(x=focal,y=abs(z)))+geom_line()+
      geom_point()+
      facet_grid(~param,scale="free_x"))


